function final_results = TEMO_MPS(fnc,opt,src_models,funcinput)

%% Algorithm Parameters
d=opt.d;
k=opt.M;
n=opt.n;
max_iter=opt.max_iter;

% GA Params
p_cross=0.9;
max_GAiter=2000;
p_size=20;
best_set_count=round(0.25*p_size);

option=opt.option;

opt.funcinput = funcinput;

% Loading Weight Vectors
if k==2
    load weightvector_2objectives
elseif k==3
    load weightvector_3objectives
end  

%% Initializing Population
x = lhsdesign(n,d);
func=str2func(fnc);
f=func(x,opt.funcinput);

%% Initializing stp object
stp=stepobj(max_iter); % Preallocating for speed
stp(1).x=x;
stp(1).f=f;  
stp(1).model.pop = [];

%% Iterating through the TEMO-MPS Algorithm
for g = 1:max_iter       
   
    fprintf('\nStep %i\n',g);       
    stp(g) = newlambda(stp(g),weights); % getting lambda
    flambda = calc_flambda(stp(g));
    stp(g) = dace(stp(g),flambda,option,src_models); % building a regression model
    
    % Getting best_set_x  
    [~,ind]=sort(flambda);
    
    % Selecting best solutions to seed GA pop.
    best_set_x = stp(g).x(ind(1:best_set_count),:);

    % Getting best available solution 
    best_y=flambda(ind(1));
    
    % Calling Steady-State GA
    best_x = StdGA(best_y,stp(g).model,best_set_x,...
        p_cross,p_size,max_GAiter);
    
    % Updating Population
    stp(g).x = [stp(g).x;best_x];    
    stp(g).f = [stp(g).f;func(best_x,opt.funcinput)];
    
    % Transfering data to StpObj of the next iteration
    stp(g+1).x=stp(g).x;
    stp(g+1).f=stp(g).f;
end

final_results = stp(end);

end