function F = fast_nondominated_sort(pop)
% ZHANG et al.: EFFICIENT APPROACH TO NONDOMINATED SORTING FOR EVOLUTIONARY
% MULTIOBJECTIVE OPTIMIZATION (2015)

[n,k]=size(pop);
F{1}=[];

% sort based on first column. if same, then based on second column and so on.
[~,ind]=sortrows(pop); 
P=pop(ind,:);

for i=1:n
    F_P=SS(P(i,:),F);   
    if F_P > size(F,1)
        F{F_P,:}=[]; % Initialize a new front 
    end
    F{F_P,:}=[F{F_P,:};P(i,:)];
end

    function dom=dominates(p1,p2)
        % Check if p1 dominates p2
        dom=false;
        if sum(p1<p2)>0 && sum(p1<=p2)==k
            dom=true;            
        end                
    end

    function front_number=SS(Pn,F)
        % Sequential Search Strategy for Finding the Front of a Solution
        x=length(F);
        kk=1;
        while true
            
            nn=size(F{kk},1);            
            if nn==0 % For empty F[k]
                front_number = kk;
                break
            else            
                for ii=linspace(nn,1,nn)
                    if dominates(F{kk}(ii,:),Pn) % Does anything in F[k] dominate Pn?
                        domi=false; 
                        break
                    else
                        domi=true;
                    end
                end

                if domi % if F[k] contains no solution dominating Pn
                    front_number = kk;
                    break
                else
                    kk=kk+1;
                    if kk>x
                        front_number=x+1;
                        break
                    end
                end 
            end
            
        end    
    end

end