classdef individual
    properties
        x
        f
    end
    methods
        function o = individual(F)
            % Initializing the individual object
            if nargin ~= 0
                n = size(F,1);
                m = size(F,2);
                if n==1 && m==1
                    o(F) = individual;
                elseif n>1 && m>1
                    o(n) = individual;
                    for i=1:n
                        o(i).x=F(i,:);
                    end
                end
            end
        end
        
        function winner = tourn_select(pop,t_size)
            % binary tournament selection to pick parents
            n=length(pop);
            ind=randsample(n,t_size);
            
            if pop(ind(1)).f > pop(ind(2)).f
                winner=ind(1);
            else
                winner=ind(2);
            end
        end
        
        function o = crossover(o,x1,x2)
            % Crossover operator as described in Self-Adaptive Genetic Algorithms
            % with Simulated Binary Crossover and similar to Abhishek's MFEA
            % GA implementation with random swapping of each variable of the two
            % child solutions.
            
            d=length(x1);
            u=rand(1,d);eta=10;
            betaq(u<=0.5)=(2*u(u<=0.5)).^(1/(eta+1));
            betaq(u>0.5)=(2*(1-u(u>0.5))).^(-1/(eta+1));
            c1=0.5*((1+betaq).*x1+(1-betaq).*x2);
            c2=0.5*((1-betaq).*x1+(1+betaq).*x2);
            
            rnvar = logical(randi(2,1,d)-1);
            o.x(rnvar)=c1(rnvar);
            o.x(~rnvar)=c2(~rnvar);
            
            o.x(o.x>1)=1;
            o.x(o.x<0)=0;
        end
        
        function o = mutate(o)
            % Mutation Operator from MO-MFEA paper
            p=o.x;
            dim=size(p,2);
            prob_mut = 1/dim;
            mum = 10;
            rnvec=p;
            for i=1:dim
                if rand(1)<prob_mut
                    u=rand(1);
                    if u <= 0.5
                        del=(2*u)^(1/(1+mum)) - 1;
                        rnvec(i)=p(i) + del*(p(i));
                    else
                        del= 1 - (2*(1-u))^(1/(1+mum));
                        rnvec(i)=p(i) + del*(1-p(i));
                    end
                end
            end
            o.x = rnvec;
        end
        
        function pop = eval_ei(pop,best_f,model)
            % function to make prediction and calculate Expected Improvement
            % a.k.a. EI.
            n=length(pop);
            for i = 1:n
                x(i,:)=pop(i).x;
            end
            
            [yhat,s] = predict(model,x);            
            
            zz=(best_f-yhat)./s;
            e_cdf = cdf('Normal',zz,0,1);
            e_pdf = pdf('Normal',zz,0,1);
            ei = (best_f-yhat).*e_cdf+s.*e_pdf;
            
            for i = 1:n
                pop(i).f=ei(i); % Using EI
            end
        end
        
    end
end